//+------------------------------------------------------------------+
//|                                                  CCI_Reversy.mq4 |
//|                               Copyright  Evgeniy Trofimov, 2009 |
//|                                   http://vkontakte.ru/id5374887/ |
//+------------------------------------------------------------------+
/*
       CCI:
:
-    (High, Low, Close)   CCIPeriod
:
-   Close,  CCI     FindCCI.
*/
#property copyright "Copyright  Evgeniy Trofimov, 2009"
#property link      "http://vkontakte.ru/id5374887/"
#define mn 20090811
extern int  CCIPeriod = 14;
extern double FindCCI = 100.0;
extern double FilterCCI = 50.0;
extern double Lots=0.1;
extern double Risk=0.03; //risk, if Lots = 0
extern int Slippage=30;
extern int TakeProfit=0;
extern int StopLoss  =0;
extern int OnlyOneEntry =1; // 0  1 -  

int      OldTicket, round;
datetime LastTime;
double   minlot, maxlot, stoplimit, spread;
//+------------------------------------------------------------------+
int init(){
   minlot = MarketInfo(Symbol(), MODE_MINLOT);
   maxlot = MarketInfo(Symbol(), MODE_MAXLOT);
   round = MathAbs(MathLog(minlot)/MathLog(10.0))+0.5; //  
   stoplimit = (MarketInfo(Symbol(), MODE_STOPLEVEL)+Slippage)*Point;
   spread = MarketInfo(Symbol(), MODE_SPREAD)*Point;
}//init()
//+------------------------------------------------------------------+
int deinit() {
   if(ObjectFind("MyLine")!=-1) ObjectDelete("MyLine");
   Comment("");
   return(0);
}//deinit()
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start() {
   double Price=PriceCCI(FindCCI);
   double CCI=iCCI(NULL,0,CCIPeriod,PRICE_TYPICAL,0);
   double OL = OpenLots(); //  OldTicket
   double WL;
   if(OL!=0) {
      OrderSelect(OldTicket,SELECT_BY_TICKET);
      if((OrderType()==OP_BUY && CCI<0) || (OrderType()==OP_SELL && CCI>0)) {
         CloseAllPozitions();
         return(0);
      }
   } else {
      WL = WaitedLots();//  OldTicket
      if(WL!=0){ 
         OrderSelect(OldTicket,SELECT_BY_TICKET);
         if((OrderType()==OP_BUYSTOP && Price<Bid) || (OrderType()==OP_SELLSTOP && Price>Ask)) {
            OrderDelete(OldTicket);
         } else {
            ModifySleep_pips(OldTicket, Price, StopLoss, TakeProfit);
         }
      } else {
         if(MathAbs(CCI)>FilterCCI) {
            if(OnlyOneEntry==1 && MathAbs(CCI)>FindCCI) return(0);
            if(Price>Ask)  BUYSleep(Price, MyLot());
            if(Price<Bid) SELLSleep(Price, MyLot());
         }
      }
   }
   return(0);
} //start()
//+------------------------------------------------------------------+
double PriceCCI(double LevelCCI, int CurrentCandle=0) {
   //  CCI
   double MovBuffer;
   double Price, SummPrice, Abs, SummAbs;
   double K = 0.015;
   int j;
   for(int i=CCIPeriod-1; i>=0; i--) {
      j=i+CurrentCandle;
      Price = (High[j]+Low[j]+Close[j])/3;
      MovBuffer=iMA(NULL,0,CCIPeriod,0,MODE_SMA,PRICE_TYPICAL,CurrentCandle);
      Abs = MathAbs(Price-MovBuffer);
      if(i>0) {
         SummPrice += Price;
         SummAbs += Abs;
      }
   }//Next i
   double CCI = (Price-MovBuffer)/((SummAbs+Abs)*K/CCIPeriod);
   Comment("CCI("+CCIPeriod+") = "+DoubleToStr(CCI,4)+"\nFindCCI = "+DoubleToStr(LevelCCI,1));
   //    CCI       
   double H = High[CurrentCandle];
   double L =  Low[CurrentCandle];
          i = CCIPeriod;
   if(CCI>=0) {
      CCI=LevelCCI;
      Price = -(H*i-L*i*i-H*i*i+L*i-CCI*H*K-CCI*L*K+3*SummPrice*i-
      CCI*3*K*SummPrice+CCI*H*K*i+CCI*L*K*i+CCI*3*K*SummAbs*i)/
      (i-i*i-CCI*K+CCI*K*i);
   } else {
      CCI=-LevelCCI;
      Price = -(H*i-L*i*i-H*i*i+L*i+CCI*H*K+CCI*L*K+3*SummPrice*i+
      CCI*3*K*SummPrice-CCI*H*K*i-CCI*L*K*i+CCI*3*K*SummAbs*i)/
      (i-i*i+CCI*K-CCI*K*i);
   }
   if(ObjectFind("MyLine")!=-1) ObjectDelete("MyLine");
   if(Price>H || Price<L) {
      ObjectCreate("MyLine", OBJ_HLINE, 0, 0,Price);
      ObjectSet("MyLine", OBJPROP_COLOR, Green); 
   } else {
      ObjectCreate("MyLine", OBJ_HLINE, 0, 0,Price);
   }
   return(Price);
}//PriceCCI()
//+------------------------------------------------------------------+
int ModifySleep_pips(int ticket, double price = 0.0, int sl = 0, int tp = 0) {
//   ,  sl  tp   
   double slPrice, tpPrice;
   if(OrderSelect(ticket, SELECT_BY_TICKET)) {
      if(sl>0 && sl*Point<stoplimit) return(0);
      if(tp>0 && tp*Point<stoplimit) return(0);
      price=NormalizeDouble(price,Digits);
      /*if(OrderStopLoss()<sl+Slippage*Point/2 && OrderStopLoss()>sl-Slippage*Point/2
      && OrderTakeProfit()<tp+Slippage*Point/2 && OrderTakeProfit()>tp-Slippage*Point/2
      && OrderOpenPrice()<price+Slippage*Point/2 && OrderOpenPrice()>price-Slippage*Point/2) return(0);*/
      if(OrderType()==OP_BUYSTOP) {
         if(sl>0) slPrice=NormalizeDouble(price-spread-sl*Point,Digits);
         if(tp>0) tpPrice=NormalizeDouble(price+tp*Point,Digits);
         if(price-Ask<stoplimit) return(0);
      } else if(OrderType()==OP_SELLSTOP) {
         if(sl>0) slPrice=NormalizeDouble(price+spread+sl*Point,Digits);
         if(tp>0) tpPrice=NormalizeDouble(price-tp*Point,Digits);
         if(Bid-price<stoplimit) return(0);
      }
      if(OrderModify(ticket, price, slPrice, tpPrice, 0)==FALSE) Print(" OrderModify(): ",GetLastError());
   }   
   return(0);
}//ModifySleep()
//+------------------------------------------------------------------+
double OpenLots(){
   //        
   double j;
   OldTicket=0;
   int total = OrdersTotal();
   for (int i = 0; i < total; i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == mn) {
         if(OldTicket==0) OldTicket=OrderTicket();
         if(OrderType()==OP_BUY) {
            j=j+OrderLots();
         } else if(OrderType()==OP_SELL) {
            j=j-OrderLots();
         }
      }
   }
   return(j);
}//OpenLots()
//+------------------------------------------------------------------+
int WaitedLots(){
   //    
   int j=0;
   OldTicket=0;
   int total = OrdersTotal();
   for (int i = 0; i < total; i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == mn) {
         if(OrderType()==OP_BUYSTOP || OrderType()==OP_SELLSTOP) {
            if(OldTicket==0) OldTicket=OrderTicket();
            j++;
         }
      }
   }
   return(j);
}//WaitedLots()
//+------------------------------------------------------------------+
int BUYSleep(double price, double lt, double sl=0, double tp=0) {
//      (   price)
   int ticket = -1; //   Ask
   Print("        ",price,", sl=",sl,", tp=",tp,", Ask=",Ask,", Bid=", Bid);
   price=NormalizeDouble(price,Digits);
   if(price-Ask<stoplimit) {
      Print("          ");
      Print("  ",stoplimit/Point, " ");
      return(0);
   }
   if(sl>0) {
      sl=NormalizeDouble(sl,Digits);
      if(price-spread-sl<stoplimit) {
         Print("BUYSleep:   StopLoss. : ",stoplimit/Point, "    Bid");
         return(0);
      }
   }
   if(tp>0) {
      tp=NormalizeDouble(tp,Digits);
      if(tp-price<stoplimit) {
         Print("BUYSleep:   TakeProfit. : ",stoplimit/Point, "    Ask");
         return(0);
      }
   }
   ticket = OrderSend(Symbol(), OP_BUYSTOP, lt, price, Slippage, sl, tp, WindowExpertName(), mn, 0, Blue); 
   if (ticket > 0) Sleep(10000);
   return(ticket);
}//BUYSleep()
//+------------------------------------------------------------------+
int SELLSleep(double price, double lt, double sl=0, double tp=0) {
//      (   price)
   int ticket = -1; //   Bid
   Print("        ",price,", sl=",sl,", tp=",tp,", Ask=",Ask,", Bid=", Bid);   
   price=NormalizeDouble(price,Digits);
   if(Bid-price<stoplimit) {
      Print("          ");
      Print("  ",stoplimit/Point, " ");
      return(0);
   }
   if(sl>0) {
      sl=NormalizeDouble(sl,Digits);
      if(sl-price-spread<stoplimit) {
         Print("SELLSleep:   StopLoss. : ",stoplimit/Point, "    Bid");
         return(0);
      }
   }
   if(tp>0) {
      tp=NormalizeDouble(tp,Digits);
      if(price-tp<stoplimit) {
         Print("SELLSleep:   TakeProfit. : ",stoplimit/Point, "    Ask");
         return(0);
      }
   }
   ticket = OrderSend(Symbol(), OP_SELLSTOP, lt, price, Slippage, sl, tp, WindowExpertName(), mn, 0, Red); 
   if (ticket > 0) Sleep(10000);
   return(ticket);
}//SELLSleep()
//+------------------------------------------------------------------+
double MyLot(int SL_pips=0) {
   if(SL_pips==0) SL_pips=10000;
   if(Lots>0) return(Lots);
   double MINLOTS = MarketInfo(Symbol(), MODE_MINLOT);
   double MAXLOTS = MarketInfo(Symbol(), MODE_MAXLOT);
   double LOTVAL  = MarketInfo(Symbol(), MODE_TICKVALUE);
   if(MAXLOTS>AccountFreeMargin()*0.99/MarketInfo(Symbol(),MODE_MARGINREQUIRED))// 0.99 -  
      MAXLOTS=AccountFreeMargin()*0.99/MarketInfo(Symbol(),MODE_MARGINREQUIRED);
   double round = MathAbs(MathLog(MarketInfo(Symbol(), MODE_LOTSTEP))/MathLog(10.0))+0.5; //  
   double lot = NormalizeDouble(AccountFreeMargin() * Risk / (SL_pips  * LOTVAL), round);
   if(lot < MINLOTS)
      lot = MINLOTS;
   else if(lot > MAXLOTS)
      lot = MAXLOTS;
   return(lot);   
}//MyLot()
//+------------------------------------------------------------------+
int CloseAllPozitions() {
   int n;
   double PriceClose;
   if(IsTradeAllowed()) {
      int  total = OrdersTotal();
      for (int i = total-1; i >= 0; i--) {
         OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
         if(OrderMagicNumber()==mn && OrderSymbol()==Symbol()) {
            if(OrderType()==OP_BUY) {
               PriceClose=MarketInfo(OrderSymbol(),MODE_BID);
            } else if(OrderType()==OP_SELL) {
               PriceClose=MarketInfo(OrderSymbol(),MODE_ASK);
            }
            n=0;
            while(IsTradeContextBusy() || !IsConnected()) {
               Sleep(2000);
               n++;
               if(n>9) break;
            }
            if(!OrderClose(OrderTicket(),OrderLots(),PriceClose,Slippage))
               Print("       ",GetLastError());
         } //  
      } // Next i
   } //   
   return(0);
}//CloseAllPozitions()
//+------------------------------------------------------------------+

